<?php

/*
 *  File to split mbox email to maildir files
 *  Author: John Carlson <johncarlson21@gmail.com>
 *  Date: 2014-03-31
 *  Version: 1.0
 *  Description:
 *  This file takes a source directory that contains files and folders from an mbox format mail client
 *  such as Thunderbird and converst it to the maildir mailbox format. 
 *  (Thunderbird is what this is based on) Other clients might be a bit different
 *  with their mbox structure.
 *  Running this on about a 1GB mbox folder took about: Total Execution Time: 0.77394656737645 mins
 *
 *  Requirements:
 *  Pear Mail_Mbox package. Found here: http://pear.php.net/package/Mail_Mbox
 *  I had to change my pear installer to reference (beta) for it to find and install it.
 *
 *  Example MBOX structure:
 *  Inbox    main inbox file
 *  Inbox.sbd     notice the .sbd
 *  Inbox.sbd/Projects     main projects file
 *  Inbox.sbd/Projects.sbd    projects folder structure to contain sub mboxes
 *  Inbox.sbd/Projects.sbd/subfolder   subfolder/mbox under projects
 *
 *  Usage: (best if run from command line)
 *  php mb2md.php
 *
 *  Change variables below for sourcedir, initialSubfolder, maildir
 */
 
// some initial settings
// uncomment these lines to turn on error logging to a local file
//error_reporting(E_ALL);
//ini_set('error_log', './errors.txt'); // change this to whatever file you like

// directory where your mbox files and folders are located
$sourcedir = "./sourcedir";
// initial subfolder to start at after main inbox. again, this might be different based on the mail client
$initialSubfolder = "./sourcedir/Inbox.sbd";
// maildirectory start
$maildir = "./Maildir/";
// mbox folder extension (.sbd), this is needed to remove the extension from the end of the folder name.
$fext = ".sbd";

// start timer
$time_start = microtime(true); 
print("Starting Conversion\n");
error_log("Starting Conversion");

require_once('Mail/Mbox.php');


/* FUNCTIONS */

function parseDir($dir,$curdir='') {
	if (is_dir($dir)) {
		$contents = scandir($dir);
		foreach($contents as $d) {
			if ($d != "." && $d != ".." && $d != ".DS_Store") {
				if (!is_dir($dir."/".$d)) { // we have a file
					processMbox($d,$dir);
				} else { // it is a dir
					parseDir($dir."/".$d);
				}
			}
		}
		
	}
}

function processMbox($file,$curdir) {
	global $maildir, $sourcedir, $initialSubfolder, $fext;
	$mdir = $maildir;
	$mbox = new Mail_Mbox($curdir."/".$file);
	$mbfile = $curdir."/".$file;
	//$mbox->setDebug(true);
	if (strtolower($file) == 'inbox') {
		// working with main inbox, set maildir to Maildir/cur/
		$mdir = $maildir."cur/";	
	} else {
		// do something with the current directory
		// make maildir directory for this directory
		$tmp = str_replace($initialSubfolder, "", $curdir);
		$tmp = str_replace($fext, "", $tmp);
		$curdir = $tmp;
		$file = str_repace($fext, "", $file);
		if (empty($tmp)) {
			// working in main directory still so need to make just a new folder with the file name
			mkdir($mdir.".".$file, 0700);
			mkdir($mdir.".".$file."/cur", 0700);
			mkdir($mdir.".".$file."/new", 0700);
			mkdir($mdir.".".$file."/tmp", 0700);
			$mdir = $mdir.".".$file."/cur/";
		} else {
			// working in a sub sub directory.
			$curdir = trim($curdir,"/");
			$parts = explode("/",$curdir);
			$nd = implode(".", $parts);
			$mdir = $mdir.".".$nd;
			error_log('nd: '.$nd);
			mkdir($mdir.".".$file, 0700);
			mkdir($mdir.".".$file."/cur", 0700);
			mkdir($mdir.".".$file."/new", 0700);
			mkdir($mdir.".".$file."/tmp", 0700);
			$mdir = $mdir.".".$file."/cur/";
			error_log('curdir: '.$curdir);
		}
		
	}
	error_log('current file: '.$file);
	error_log('current maildir: '.$mdir);
	error_log('current mbox directory: '.$curdir);
	print('Converting: '.$mbfile." to ".$mdir."\n");
	$mbox->open();
	error_log('total messages: '.$mbox->size());
	for ($n=0; $n < $mbox->size(); $n++) {
		$message = $mbox->get($n);
		$filename = $mdir.time();
		//inc id
		$inc = "000000";
		$len = strlen($n);
		$inc = substr($inc, 0, -$len);
		$inc = $inc.$n;
		$filename .= ".".$inc;
		$filename .= ".mbox";
		$filename .= ":2,Sa"; // this means the message has been seen.
		error_log('saving file to: '.$filename);
		file_put_contents($filename,$message);
		//sleep(1);
	}

	$mbox->close();
}// end processMbox


parseDir($sourcedir);

// finish time
$time_end = microtime(true);
$execution_time = ($time_end - $time_start)/60;
error_log('Total Execution Time: '.$execution_time.' mins');
print('Total Execution Time: '.$execution_time.' mins.\n');
exit();
?>
